package handler

import (
	"database/sql"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/interfaces"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
	"github.com/gofiber/fiber/v2"
)

const (
	maxArticlesCount = 3
	articleParam     = "article"
	articleCacheKey  = "article"
)

func ArticleHandler(sp interfaces.ServiceProvider) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		articleReq := fctx.Params(articleParam)

		renderData, found := sp.GetCacheService().Get(articleCacheKey + articleReq)

		if !found {
			article, err := sp.GetArticleRepository().GetByURL(ctx, articleReq)
			if err != nil {
				if err == sql.ErrNoRows {
					return fiber.ErrNotFound
				}
				return err
			}

			if !article.IsActive {
				return fiber.ErrNotFound
			}

			articleDTO := mapper.ConvertArticleModelToDTO(*article)

			// All used tags
			tags, err := sp.GetTagRepository().GetAllUsed(ctx)
			if err != nil {
				return err
			}

			tagsDTO := mapper.ConvertTagModelsToDTO(tags)

			// Last articles
			articles, err := sp.GetArticleRepository().GetAllPreview(ctx)
			if err != nil {
				return err
			}
			if len(articles) > maxArticlesCount {
				articles = articles[:maxArticlesCount]
			}

			renderData = fiber.Map{
				"headTitle":       "От слона к суслику - статьи про PHP, Go, алгоритмы",
				"headDescription": articleDTO.MetaDescription,
				"headKeywords":    articleDTO.MetaKeywords,
				"pageTitle":       "Статья<br>" + articleDTO.Title,
				"article":         articleDTO,
				"sidebarArticles": mapper.ConvertArticlePreviewModelsToDTO(articles),
				"sidebarTags":     tagsDTO,
			}

			sp.GetCacheService().Set(articleCacheKey+articleReq, renderData)
		}

		return fctx.Render("article", renderData, "_layout")
	}
}
