package handler

import (
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/interfaces"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
	"github.com/gofiber/fiber/v2"
)

const allPreviewArticlesCacheKey = "all-preview-articles"

func MainPageHandler(sp interfaces.ServiceProvider) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		renderData, found := sp.GetCacheService().Get(allPreviewArticlesCacheKey)

		if !found {
			articles, err := sp.GetArticleRepository().GetAllPreview(fctx.Context())
			if err != nil {
				return err
			}

			renderData = fiber.Map{
				"headTitle":       "От слона к суслику - статьи про PHP, Go, алгоритмы",
				"headDescription": "список статей",
				"headKeywords":    "Дмитрий Гнатенко, программист, PHP, Go, Golang, программирование, статьи, блог",
				"pageTitle":       "Список статей",
				"articles":        mapper.ConvertArticlePreviewModelsToDTO(articles),
			}

			sp.GetCacheService().Set(allPreviewArticlesCacheKey, renderData)
		}

		return fctx.Render("index", renderData, "_layout")
	}
}
