package handler

import (
	"database/sql"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/interfaces"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
	"github.com/gofiber/fiber/v2"
)

const tagParam = "tag"
const tagCacheKey = "tag"

func TagHandler(sp interfaces.ServiceProvider) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		tagReq := fctx.Params(tagParam)

		renderData, found := sp.GetCacheService().Get(tagCacheKey + tagReq)

		if !found {
			tag, err := sp.GetTagRepository().GetByURL(ctx, tagReq)
			if err != nil {
				if err == sql.ErrNoRows {
					return fiber.ErrNotFound
				}
				return err
			}

			articles, err := sp.GetArticleRepository().GetPreviewByTagID(ctx, tag.ID)
			if err != nil {
				return err
			}

			renderData = fiber.Map{
				"headTitle":       "От слона к суслику - статьи про PHP, Go, алгоритмы",
				"headDescription": "статьи с тегом " + tag.Tag,
				"headKeywords":    "программирование, статьи, блог, " + tag.Tag,
				"pageTitle":       "Тег<br>" + tag.Tag,
				"articles":        mapper.ConvertArticlePreviewModelsToDTO(articles),
			}

			sp.GetCacheService().Set(tagCacheKey+tagReq, renderData)
		}

		return fctx.Render("tag", renderData, "_layout")
	}
}
